local addon = LibStub("AceAddon-3.0"):NewAddon("Langoustine","AceEvent-3.0","AceConsole-3.0","LibSink-2.0","AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Langoustine",false)

local defaults = {
	profile = {
		MTShow=true,
		MTOptions = {
			x=0, 
			y=0, 
			locked=true, 
			maxTanks = 10, 
			unitHeight=10, 
			unitWidth=10, 
			font="", 
			barTexture="Blizzard", 
			growth="Down", 
			icon=true, 
			alpha=1,  
			showAggro=true,
			colorByClass=true,
			backDrop="",
			scale=1,
			inverseHealth = false,
			enableClickCast = true,
			reverseOrder=false,
		},
		MTShowTarget=true,
		MTShowTargetTarget=true,
		AutoPromote={},
		AutoLoot={ method="master", threshold="4", looter=""},
		AutoPromoteEnabled=true,
		AutoLootEnabled=true,
		Minimap = {},
		Sink={},
		NotifyMTDeath=true,
		WhisperInviteTriggger="invite",
		WhisperInvite=true,
		ReadyCheck={},
	},
}
local guildRanks = {}
local guildMembers = {}
local wipe,tinsert = wipe, tinsert

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("LangoustineDB", defaults, "Default")
	self:RegisterEvent("RAID_ROSTER_UPDATE","RaidCheck")
	self:RegisterEvent("GUILD_ROSTER_UPDATE","GuildCheck")
end

-- when we disable ourselves
function addon:OnDisable()
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self.autoSetML = false
	self.autoSetLoot = false
end

-- enable ourselves
function addon:OnEnable()
	self:RegisterEvent("CHAT_MSG_WHISPER","CheckInvite")
end

-- update local rank and member list
function addon:GuildCheck()
	wipe(guildRanks)
	wipe(guildMembers)	
	for i = 1, GuildControlGetNumRanks() do
		tinsert(guildRanks, GuildControlGetRankName(i))
	end
	local numGuildMembers = GetNumGuildMembers()
	for i = 1, numGuildMembers do
		local name = GetGuildRosterInfo(i)
		if name then
			guildMembers[name] = true
		end
	end
end

local raiders = {}
-- conver the party to raid
function addon:ConvertToRaid()
	if GetNumPartyMembers() > 0 and not UnitInRaid("player") then
		self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
		ConvertToRaid()
	end
end

-- disband the raid
function addon:Disband()
	if (isRaidOfficer()) then
		local player = UnitName("player")
		for i = 1, GetNumRaidMembers() do
			local name = GetRaidRosterInfo(i)
			if name ~= player then
				UninviteUnit(name)
			end
		end
		LeaveParty()
		self:Disable()
	end
end

-- invite by guild rank or by level
function addon:Invite(byRank,byLevel)
	self:RegisterEvent("PARTY_MEMBERS_CHANGED","ConvertToRaid")
	wipe(peopleToInvite)
	for i = 1, GetNumGuildMembers() do
		local name, _, rank, level, _, _ , _, _, online = GetGuildRosterInfo(i)
		-- check for both party and raid since we may run invite after grabbign a few people
		if name and online and not UnitInParty(name) and not UnitInRaid(name) then
			if byLevel and level <= byLevel then
				tinsert(raiders, name)
			elseif byRank and (rank + 1) <= byRank then
				tinsert(raiders, name)
			end
		end
	end
	-- invite everyone
	for i,name in ipairs(raiders) do
		InviteUnit(name)
	end
	wipe(raiders)
end

-- Raid Checks
function addon:RaidCheck()
	-- Are we in a raid?
	local inRaid = UnitInRaid("player")
	local _, instanceType = IsInInstance()
	if instanceType == "pvp" or instanceType == "arena" then return end
	-- if we are not enabled and joined a raid enable ourselves
	if not self:IsEnabled() and inRaid then
		self:Enable()
	elseif self:IsEnabled() and not inRaid then -- left the raid group disable ourselves
		self:Disable()
	end
	-- Raid leader checks
	if (isRaidLeader()) then
		-- Check promote list
		if self.db.profile.AutoPromoteEnabled then
			for i = 1, GetNumRaidMembers() do
				local name, rank = GetRaidRosterInfo(i)
				if name and rank == 0 and self.db.profile.AutoPromote[name] then
					PromoteToAssistant(name)
				end
			end
		end
		-- Check Loot setup
		if self.db.profile.AutoLootEnabled then
			local method,_,_ = GetLootMethod()
			if method ~= self.db.profile.AutoLoot.method and not self.autoSetLoot then
				SetLootMethod(method,self.db.profile.AutoLoot.looter,self.db.profile.AutoLoot.threshold)
				SetLootThreshold(self.db.profile.AutoLoot.threshold)
				self.autoSetLoot = true
			end
			method,_,masterLooter = GetLootMethod()
			if method == "master" and not self.autoSetML then
				-- do we have the right master looter?
				local looterName = GetRaidRosterInfo(masterLooter)
				if looterName ~= self.db.profile.AutoLoot.looter then
					-- is the looter we want in the raid?
					for i = 1, GetNumRaidMembers() do
						local name, rank = GetRaidRosterInfo(i)
						if name == self.db.profile.AutoLoot.looter then
							SetLootMethod(method,name,self.db.profile.AutoLoot.threshold)
							self.autoSetML = true
							-- we set the ml according to prefs, stop checking anymore till we leave the raid
							-- during a run the raidleader may reset loot as he wants and/or the master looter
						end
					end
				end
			end
		end
	end
end